@@ -1,6 +1,7 @@
 from .viewsets import GeneralViewSet, GeneralListSerializer, GeneralSerializer
 from . import auth
 from .core import parse_model, has_model_attr, ModelInfo
+from .pagination import PagePagination
 from rest_framework.routers import SimpleRouter
 from rest_framework import serializers
 from django.urls.conf import path
@@ -54,6 +55,10 @@ class DefaultSite():
             setattr(admin_class, name, ())
         return admin_class(model, self)
     
+    def _build_pagination_class(self, admin):
+        pagination_class = type('', (PagePagination,), {'page_size': admin.list_per_page})
+        return pagination_class
+    
     def _build_list_serialize_class(self, admin):
         args = {
             'pk': serializers.CharField(),
@@ -93,7 +98,7 @@ class DefaultSite():
         self._patch_fields(admin, args)
         for field in admin.opts.get_fields():
             t = type(field)
-            if t in (ManyToManyField, ManyToOneRel):
+            if t in (ManyToManyField,):
                 args[field.name] = serializers.StringRelatedField(many=True)
             elif t in (ForeignKey,):
                 args[field.name] = serializers.StringRelatedField(many=False)
@@ -175,6 +180,7 @@ class DefaultSite():
             viewset_class.autocomplete_serializer_class = self._build_autocomplete_serialize_class(admin)
             viewset_class.serializer_class = self._build_serialize_class(admin)
             viewset_class.view_serializer_class = self._build_view_serialize_class(admin)
+            viewset_class.pagination_class = self._build_pagination_class(admin)
             self._build_actions(viewset_class)
             self._registry[model] = admin
             self.router.register(info.prefix, viewset_class)
