@@ -14,12 +14,51 @@ from django.contrib.admin.options import ModelAdmin
 from django.contrib import messages
 from django.db.models import fields
 from .core import get_passwords, ModelInfo
+from . import log
 from rest_framework.permissions import BasePermission
 
 class GeneralMetadata(SimpleMetadata):
     _view = None
     def __init__(self):
         pass
+    def _get_filters(self, view):
+        admin = view._admin
+        list_filter = []
+        for key in admin.list_filter:
+            field = admin.opts.get_field(key)
+            title = field.verbose_name
+            choices = field.choices
+            if not choices:
+                choices = []
+                t = type(field)
+                if t == fields.related.ManyToManyField:
+                    for k, v in field.get_choices():
+                        if k:
+                            choices.append((k, v))
+                elif t == fields.BooleanField:
+                    choices = (
+                        (1, '是'),
+                        (0, '否'),
+                    )
+            list_filter.append({
+                'key': key,
+                'label': title,
+                'choices': choices,
+            })
+        search_fields = []
+        for key in admin.search_fields:
+            field = admin.opts.get_field(key)
+            title = field.verbose_name
+            search_fields.append({
+                'key': key,
+                'label': title,
+            })
+        rs = {
+            'listFilter': list_filter,
+            'searchFields': search_fields,
+        }
+        return rs
+
     def _patch_input_type(self, field, data):
         def is_password_type(admin, fieldname):
             for name, _ in admin.passwords:
@@ -37,6 +76,8 @@ class GeneralMetadata(SimpleMetadata):
         data = SimpleMetadata.determine_metadata(self, request, view)
         data['fieldsets'] = view._admin.get_fieldsets(request)
         data['addFieldsets'] = view._admin.add_fieldsets
+        data['filters'] = self._get_filters(view)
+        data['pageSize'] = self._view._admin.list_per_page
         return data
     
     def get_field_info(self, field):
@@ -86,6 +127,10 @@ class GeneralPermission(BasePermission):
 
 class GeneralSerializer(serializers.ModelSerializer):
     _admin: ModelAdmin = None
+    @property
+    def request(self):
+        return self.context['request']
+    
     def _del_file(self, instance):
         def filter_del_keys():
             prefix = '__del__'
@@ -109,13 +154,35 @@ class GeneralSerializer(serializers.ModelSerializer):
                 func = getattr(self._admin, func)
             validated_data[k] = func(validated_data, validated_data[k])
     def update(self, instance, validated_data):
+        def modify_fields():
+            rs = []
+            for k, v in validated_data.items():
+                oldv = getattr(instance, k)
+                if oldv != v:
+                    rs.append(k)
+            return rs
         self._del_file(instance)
         self._set_password(validated_data)
-        return serializers.ModelSerializer.update(self, instance, validated_data)
+        fields = modify_fields()
+        obj = serializers.ModelSerializer.update(self, instance, validated_data)
+        message = []
+        if len(fields) > 0:
+            item = {
+                'changed': {
+                'fields': fields,
+                }
+            }
+            message.append(item)
+        log.log_change(self.request, obj, message)
+        return obj
     
     def create(self, validated_data):
         self._set_password(validated_data)
-        return serializers.ModelSerializer.create(self, validated_data)
+        obj = serializers.ModelSerializer.create(self, validated_data)
+        message = [{'added': {}}]
+        log.log_addition(self.request, obj, message)
+        return obj
+    
 
 class GeneralListSerializer(serializers.ModelSerializer):
     _admin: ModelAdmin = None
@@ -140,6 +207,8 @@ class GeneralViewSet(viewsets.ModelViewSet):
     
  
     def _inner_delete(self, request, queryset):
+        for obj in queryset:
+            log.log_deletion(request, obj, str(obj))
         count, _ = queryset.delete()
         message = f'{count}条记录被删除'
         self._admin.message_user(request, message, messages.SUCCESS)
@@ -212,46 +281,6 @@ class GeneralViewSet(viewsets.ModelViewSet):
             rs.append(column)
         return rs
     
-    @action(detail=False)
-    def filters(self, _):
-        list_filter = []
-        for key in self._admin.list_filter:
-            field = self._model._meta.get_field(key)
-            title = field.verbose_name
-            choices = field.choices
-            if not choices:
-                choices = []
-                t = type(field)
-                if t == fields.related.ManyToManyField:
-                    for k, v in field.get_choices():
-                        if k:
-                            choices.append((k, v))
-                elif t == fields.BooleanField:
-                    choices = (
-                        (1, '是'),
-                        (0, '否'),
-                    )
-                # for item in self._model.objects.values_list(key, flat=True).distinct():
-                #     choices.append((item, f'{item}'))
-            list_filter.append({
-                'key': key,
-                'label': title,
-                'choices': choices,
-            })
-        search_fields = []
-        for key in self._admin.search_fields:
-            field = self._model._meta.get_field(key)
-            title = field.verbose_name
-            search_fields.append({
-                'key': key,
-                'label': title,
-            })
-        rs = {
-            'list_filter': list_filter,
-            'search_fields': search_fields,
-        }
-        return Response(rs)
-
     def list(self, request, *args, **kwargs):
         self.serializer_class = self.list_serializer_class
         data = viewsets.ModelViewSet.list(self, request, *args, **kwargs)
@@ -274,4 +303,7 @@ class GeneralViewSet(viewsets.ModelViewSet):
     def str(self, request, *args, **kwargs):
         instance = self.get_object()
         return Response({'str': instance.__str__()})
-    
\ No newline at end of file
+    
+    def perform_destroy(self, instance):
+        log.log_deletion(self.request, instance, str(instance))
+        viewsets.ModelViewSet.perform_destroy(self, instance)
